#!/bin/bash
###########################################################
# 
# usage: convert_jump.sh [/tmpdirectory] [3x3] [256x192] [image1] ... [image9]
# 
# script to convert the selected image to mpeg2 still
# for vdr image plugin 
# needs : netpbm-progs, mjpegtools [and a vdr or atleast a dvb ff card and the dvb driver tool test_stillimage]
#
# History:
# 2004-07-17    a.brachold - use ppmlabel for numbers on index instead of pbmindex
# 2004-06-27    a.brachold - remove ImageMagick stuff
#                            Remove the last script step copy xxx.mpg to image.mpg
# 2004-06-22    a.brachold - cleanup logging 
###########################################################
[ "z$DEBUG" = "zyes" ] && set -xv

# save parameters for logging
CALLED="CALLED '$0 $*'"

INC_DIR=`dirname $0`
if [ ! -x ${INC_DIR}/convert_functions.sh ] ; then
   echo "Can't find or execute include file convert_functions.sh"
   exit 1
fi
. ${INC_DIR}/convert_functions.sh

CONVERT_TEMPDIR="${1:-"/tmp/image"}"
convert_init $CONVERT_TEMPDIR $CALLED

DIR=$(dirname "$4")
FILE=$(basename "$4")
NEWDIR="${CONVERT_TEMPDIR}/${DIR}"
NEWFILE="${NEWDIR}/${FILE}Z.mpg"
JUMPFILE="${NEWDIR}/JUMP.pnm"
logit "FILE: $FILE"
logit "SRCFILE1: $4"
logit "SRCFILE2: $5"
logit "SRCFILE3: $6"
logit "SRCFILE4: $7"
logit "SRCFILE5: $8"
logit "SRCFILE6: $9"
logit "SRCFILE7: ${10}"
logit "SRCFILE8: ${11}"
logit "SRCFILE9: ${12}"
logit "JUMPFILE: $JUMPFILE"

  [ ! -d "$NEWDIR" ] && mkdir -p "$NEWDIR" | tee -a "$CONVERT_LOGFILE"

  index=1
  for i in "$4" "$5" "$6" "$7" "$8" "$9" "${10}" "${11}" "${12}" ; do
     # Check for Image 
     if [ ! -s "$i" ]; then
         cp -l "$DUMMY_TILE" "${CONVERT_TEMPDIR}/${index}"
      else
        getpnmtool "$i"
        $TO_PNM "$i" | pnmscale -xysize 256 192 |\
          pnmcomp -align=center -valign=middle - "$DUMMY_TILE" \
            > "${CONVERT_TEMPDIR}/${index}"

        if [ "$SHOW_NUMBERS" = "ztrue" ] ; then
## Method 1 ppmlabel, draw four times Black and two times white Number 1...9
          cat "${CONVERT_TEMPDIR}/${index}" | \
            ppmlabel -x 204 -y 44 -background transparent -colour black -size 24 -text "$index" | \
              ppmlabel -x 204 -y 47 -background transparent -colour black -size 24 -text "$index" | \
                ppmlabel -x 206 -y 44 -background transparent -colour black -size 24 -text "$index" | \
                  ppmlabel -x 206 -y 47 -background transparent -colour black -size 24 -text "$index" | \
                    ppmlabel -x 205 -y 44 -background transparent -colour white -size 24 -text "$index" | \
                      ppmlabel -x 205 -y 45 -background transparent -colour white -size 24 -text "$index"  \
                        > "${CONVERT_TEMPDIR}/${index}.tmp" && \
          mv -f "${CONVERT_TEMPDIR}/${index}.tmp" "${CONVERT_TEMPDIR}/${index}"  
       
## Method 2 pbmtext => Only sweet with own fontfile see pbmtext
#         INDEX_TILE="$CONVERT_TEMPDIR/overlay-$index.pnm"
#         [ ! -s "$INDEX_TILE" ] && pbmtext -builtin fixed $index > "$INDEX_TILE" 
#         [ ! -s "$INDEX_TILE" ] && logit "Error $INDEX_TILE does not exist,\n it used as overlay index file" && exit 1
#         cat "${CONVERT_TEMPDIR}/${index}" | \
#           pnmcomp -invert -alpha "$INDEX_TILE" -align=center -valign=middle "$INDEX_TILE" -    \
#             > "${CONVERT_TEMPDIR}/${index}.tmp" && \
#         mv -f "${CONVERT_TEMPDIR}/${index}.tmp" "${CONVERT_TEMPDIR}/${index}"  
##
        fi
      fi
      # Image not found, use link to black tile
      if [ ! -s "${CONVERT_TEMPDIR}/${index}" ]; then
        cp -l "$DUMMY_TILE" "${CONVERT_TEMPDIR}/${index}"
      fi

      let index++
    done

   # Merge image 
   cd "$CONVERT_TEMPDIR"
   pnmcat -black -leftright 1 2 3 > row1
   pnmcat -black -leftright 4 5 6 > row2
   pnmcat -black -leftright 7 8 9 > row3
   pnmcat -black -topbottom row1 row2 row3 > "$JUMPFILE"
   rm -f row1 row2 row3
   rm -f 1 2 3 4 5 6 7 8 9

# $JUMPFILE now contains the 3x3 matrix in pnm format.
# TODO: Make sure resolution is correct 


# This is needed for aspect and rotation
getresolution "$JUMPFILE"
convert_imagetompeg "$JUMPFILE" "$NEWFILE" "$FLIP"
rm -f "$JUMPFILE"

# NEWFILE contains the 3x3 matrix file in mpeg
if [ ! -s "$NEWFILE" ] ; then 
  logit "$DATE: ERROR Convert $IMAGEFILE to STILL MPEG2 $NEWFILE not successful"; 
  exit 1
fi

logit "$DATE: $NEWFILE created successfull ...";
exit 0
