#!/bin/bash
################################################################################
#
# convert_zoom.sh [image] [/tmpdirectory] [Original] [X_RES_ZOOM] [Y_RES_ZOOM] [X_POS_ZOOM] [Y_POS_ZOOM]
#
# script to convert the selected image to mpeg2 still
# for vdr image plugin 
# needs : netpbm-progs, mjpegtools [and a vdr or atleast a dvb ff card and the dvb driver tool test_stillimage]
# for best and fastes extra tools it's still suggested to use ImageMagick
#
# History:
# 2004-07-26  a.brachold - change mkdir -v -p to mkdir -p, for work with busybox
# 2004-06-27  a.brachold - remove ImageMagick stuff
#                            Remove the last script step copy xxx.mpg to image.mpg
# 2004-06-22  a.brachold - cleanup logging 
################################################################################
[ "z$DEBUG" = "zyes" ] && set -xv

# save parameters for logging
CALLED="CALLED '$0 $*'"


INC_DIR=`dirname $0`
if [ ! -x ${INC_DIR}/convert_functions.sh ] ; then
   echo "Can't find or execute include file convert_functions.sh"
   exit 1
fi
. ${INC_DIR}/convert_functions.sh

if [ $# -ne 7 ] ; then
   logit "Error, wrong number of arguments!"
   exit 1
fi


SRCFILE="$1"
CONVERT_TEMPDIR="${2:-"/tmp/image"}"

# set defaults
convert_init $CONVERT_TEMPDIR "$CALLED"

if test "$3" == "Right"; then ROTATE="YES"; FLIP="pnmflip -rotate270"; fi
if test "$3" == "Left";  then ROTATE="YES"; FLIP="pnmflip -rotate90";  fi
if test "$3" == "Original"; then ORIGINAL="YES"; fi

X_RES_ZOOM="$4"
Y_RES_ZOOM="$5"
X_POS_ZOOM="$6"
Y_POS_ZOOM="$7"

DIR=$(dirname "$SRCFILE")
FILE=$(basename "$SRCFILE")
logit "FILE: $FILE"
NEWDIR="${CONVERT_TEMPDIR}/${DIR}"
NEWFILE="${NEWDIR}/${FILE}Z.mpg"

ZOOMFILE="${NEWDIR}/${FILE}Z.jpg"
logit "SRCFILE : $SRCFILE"
logit "ZOOMFILE: $ZOOMFILE"
logit "X_RES_ZOOM: $X_RES_ZOOM"
logit "Y_RES_ZOOM: $Y_RES_ZOOM"
logit "X_POS_ZOOM: $X_POS_ZOOM"
logit "Y_POS_ZOOM: $Y_POS_ZOOM"

   #using netpbm pnmcut
   logit "using netpbm tools for zoom"
   ZOOMFILE="${NEWDIR}/${FILE}Z.pnm"
   getpnmtool "$SRCFILE"

   if [ "z$USE_ZOOMCACHE" = "ztrue" ] ; then
      ZOOMCACHE="$CONVERT_TEMPDIR/${SRCFILE}.pnm"
      CACHEFILE="$CONVERT_TEMPDIR/pnmcache.lastfile"

      touch "$CACHEFILE"
      if [ ! "z`cat $CACHEFILE`" = "z$ZOOMCACHE" -o ! -e "$ZOOMCACHE" ] ; then
         rm -f "`cat $CACHEFILE`"                   # delete old cache
         logit "creating zoomcache for $SRCFILE"
         $TO_PNM "$SRCFILE" > "$ZOOMCACHE"
         echo "$ZOOMCACHE" > "$CACHEFILE"
      fi
      logit "using zoomcache for $SRCFILE"
      cat "$ZOOMCACHE" | \
        pnmcut -pad -left $X_POS_ZOOM -width $X_RES_ZOOM -top $Y_POS_ZOOM -height $Y_RES_ZOOM > "$ZOOMFILE"

   else 
      logit "not using zoomcache for $SRCFILE"
      logit "$TO_PNM $SRCFILE | pnmcut -left $X_POS_ZOOM -width $X_RES_ZOOM -top $Y_POS_ZOOM -height $Y_RES_ZOOM > $ZOOMFILE"
      $TO_PNM "$SRCFILE" | \
        pnmcut -pad -left $X_POS_ZOOM -width $X_RES_ZOOM -top $Y_POS_ZOOM -height $Y_RES_ZOOM > "$ZOOMFILE"
   fi

# This is needed for rotating and aspect corretion
getresolution "$ZOOMFILE"


[ ! -d "$NEWDIR" ] && mkdir -p "$NEWDIR" | tee -a "$CONVERT_LOGFILE"

convert_imagetompeg "$ZOOMFILE" "$NEWFILE" "$FLIP"
rm -f "$ZOOMFILE"

if [ ! -s "$NEWFILE" ] ; then 
  logit "$DATE: ERROR Convert $IMAGEFILE to STILL MPEG2 $NEWFILE not successful"; 
  exit 1
fi

logit "$DATE: $NEWFILE created successfull ...";
exit 0
